part of '../../../git_assistant.dart';

/// Git log command class.
///
/// Provides log command options in simple way and recursive to be more like CLI.
///
/// ```dart
/// var git = Git();
/// print(git.log.prettyOpt(PF.full)); // 'git log --pretty=full'
/// git.log.exec(); // executes 'git log' and returns its output
/// ```
class LogGitChain extends BaseGitChain {
  LogGitChain(String chainName, BaseGitChain prevChain)
      : super(chainName, prevChain);

  LogGitChain _createLogOption(String chainName) {
    return LogGitChain(chainName, this);
  }

  /// Generates --pretty='format' or tformat:'format' option
  ///
  /// Takes one parameter which can be:
  /// 1. [PF] instances
  /// 2. or combination of [PP] instances and [String]
  ///
  /// ## Using [PF] for format option
  /// ```dart
  /// var git = Git();
  /// git.log.prettyOpt(PF.oneline).exec();
  /// print(git.log.prettyOpt(PF.email));
  /// git.log.prettyOpt(PF.fuller).exec();
  /// ```
  ///
  /// ## Using combination of [PP] and [String]
  /// ```dart
  /// var git = Git();
  /// // Executes 'git log --pretty=tformat:"commit %H"'
  /// git.log.prettyOpt('commit ${PP.commitHash}').exec();
  /// ```
  LogGitChain prettyOpt(dynamic format) {
    late String prettyChainName;
    if (format is PF) {
      prettyChainName = format.toString();
    } else if (format is String) {
      prettyChainName = 'tformat:$format';
    } else {
      throw Exception(
        '[LogGitChain.prettyOpt]: "format" should String or PF enum instance',
      );
    }
    return _createLogOption('--pretty=$prettyChainName');
  }

  /// Generates -{count of last commit} option
  ///
  /// For example:
  /// ```sh
  /// git log -5 --pretty=full
  /// ```
  /// Shows you last five commits and in your code will be:
  /// ```dart
  /// var git = Git();
  /// print(git.log.atLeast(2).prettyOpt(PF.full).exec());
  /// ```
  LogGitChain atLeast(int count) {
    return _createLogOption('-$count');
  }
}
